unit uThread;

interface

uses
  Classes, Windows, SysUtils, StrUtils, uUtils, Messages;

type
  {==============================================================================}
  TThreadRC5 = class(TThread)
    private
      { Dclarations prives }
      bin: string;                // reprsentation binaire de la trame
      RC5Data: TRC5Data;          // structure qui va contenir nos infos sur la trame
      CopyData: TCopyData;        // structure qui va tre envoy avec le message WM_COPYDATA et qui va contenir un pointeur sur la structure TRC5data
      hHandle: THandle;           // handle de la fiche qui rceptionne le message WM_COPYDATA
      hCom: THandle;              // handle du port COM
    protected
      { Dclarations protges }
      procedure Analyse;          // procedure de surveillance du port COM
      procedure Send;             // va envoyer le message WM_COPYDATA avec ses infos
    public
      { Dclarations publiques }
      constructor Create(AHandle, ACom: THandle);
      procedure Execute; override;
    end;
  {==============================================================================}
implementation

{==============================================================================}
{   Constructeur de notre thread                                               }
{==============================================================================}
constructor TThreadRC5.Create(AHandle, ACom: THandle);
begin
  inherited Create(False);        // execution du thread lors de sa cration
  hHandle := AHandle;             // handle de la fiche Form1 (pour le message WM_COPYDATA)
  hCom := ACom;                   // handle du port COM

  FreeOnTerminate := True;
  priority := tpHigher;
end;
{==============================================================================}
{   Lance la surveillance du port                                              }
{==============================================================================}
procedure TThreadRC5.Execute;
begin
  while not Terminated do
    begin
      Analyse;                    // on lance la surveillance
    end;
end;
{==============================================================================}
{   Mise  jour des donnes et dclenchement de l'venement de rception       }
{==============================================================================}
procedure TThreadRC5.Send;
begin
  { Structure TRC5Data }
  with RC5Data do
    begin 
      Code := bin;                                    // reprsentation binaire
      Adresse := AnsiMidStr(bin,4,5);                 // extraction de l'adresse
      Commande := AnsiMidStr(bin,9,6);                // extraction de la commande
      BitControl := strToInt(bin[3]);                 // extraction du bit de rptition
      CommandeHexa := BinToHex(AnsiMidStr(bin,9,6));  // conversion de la commande sous sa forme hexadcimale
    end;
  { Structure TCopyData }
  with CopyData do
    begin
     dwData := 1;
     cbData := sizeof(RC5Data);
     lpData := @RC5Data;                              // pointeur sur la structure TRC5Data
    end;

  // on envoie le message WM_COPYDATA avec un pointeur sur notre structure TRC5Data en paramtre
  SendMessage( hHandle, WM_COPYDATA, 0, LongInt(@CopyData));

end;
{==============================================================================}
{   Attend et gre la rception du signal                                      }
{==============================================================================}
procedure TThreadRC5.Analyse;
var
  i, j, tps : integer;
	bBits : boolean;
	nbCom : integer;
  Errors : DWord;
	PerformanceCount, PerformanceCountOld, Frequency : TLargeInteger;
  szCom : array [0..256] of char;
  szComOld : array [0..256] of char;
begin
  i := 2;       // les 2 premires impulsions du bit de start
  j := 0;       // index des tableaux de bits
  nbCom := 0;   // nombre d'venements sur ligne DSR

  SetCommMask(hCom,EV_DSR);                       // on applique un masque sur le changement d'tat de DSR
  QueryPerformanceFrequency(Frequency);           // on rcupre la frquence du compteur haute performance
  while not Terminated do                         // tant que le thread est activ
    begin
      WaitCommEvent(hCom,Errors,nil);             // on attend les venements DSR
      inc(nbCom);                                 // on incrmente le nombre d'impulsions (vnements)
      PerformanceCountOld := PerformanceCount;
      QueryPerformanceCounter(PerformanceCount);  // on rcupre la valeur courante du compteur haute performance
      Tps := Trunc((PerformanceCount-PerformanceCountOld)*1000000/Frequency); // temps coul en s: (t1 - t0) * 1000000 / frequence.  (1s = 1000000s)

      if (Tps > 13335)then                        // si Tps >  la longueur de 3/4 de bits * 10 (1 bits = 1778 s --> 1778*0.75 = 1333.5 * 10 = 13335)
      begin
        nbCom :=1;                                // on rinitialise  1
        i :=2;                                    // toujours notre bit de start
        j :=0;                                    // l'index de tableau  0
        FillChar(szCom, sizeof(szCom),0);         // on remplit notre tableau de bits actuel de 0
        FillChar(szComOld, sizeof(szComOld),0);   // on remplit notre ancien tableau de bits de 0
      end;

      if nbCom = i then                           // si on as notre bit de start ou qu'il n'y a pas eu de changement d'tat pour le bit prcdent
      begin
        bBits :=(nbCom mod 2 =0);                 // on vrifie si nbCom est un multiple de 2
        if bBits then szCom[j]:= chr($31) else  szCom[j]:= chr($30);  // si oui alors on met le bit  1 sinon 0
        inc(j);                                   // on incremente l'index du tableau
        if(Tps > 1333) then                       // si Tps >  la longueur de 3/4 de bits c'est qu'il n'y a pas eu de changement d'tat
          begin
            if not bBits then szCom[j]:= chr($31) else  szCom[j]:= chr($30); // et que ce n'est pas un multiple de 2 alors 1 sinon 0
            inc(i);
          end
        else
          begin
            inc(i,2);                              // on incremente de 2
            if not bBits then szCom[j]:= chr($30); // si Tps est infrieur  3/4 de bits et non multiple de 2 alors 0
          end;
      end;

      // si on a nos 14 bits et que notre tableau est diffrent du prcdent
      if (strlen(szCom)=14) and (strcomp(szCom,szComOld)<>0) then
        begin
          strcopy(szComOld,szCom);  // on sauvegarde notre tableau de bits
          bin := string(szComOld);  // on rcupre notre reprsentation binaire
          synchronize(Send);        // et enfin on dclenche l'venement de rception
          Sleep(5);                 // une petite pause de 5 ms
        end;

    end;
end;

end.
